styles = {
    "alignment": {
        "color": "black",
        "linestyle": "-",
        "linewidth": 1.0,
        "alpha": 0.2,
        "label": "raw alignment"
    },
    "despiked": {
        "color": "blue",
        "linestyle": "-",
        "linewidth": 1.4,
        "alpha": 1.0,
        "label": "de-spiked alignment"
    },
    "line1": {
        "color": "black",
        "linestyle": "-",
        "linewidth": 1.2,
        "alpha": 0.3,
        "label": "line1"
    },
    "line2": {
        "color": "blue",
        "linestyle": "-",
        "linewidth": 1.4,
        "alpha": 1.0,
        "label": "line2"
    },
    "line3": {
        "color": "green",
        "linestyle": "-",
        "linewidth": 1.4,
        "alpha": 1.0,
        "label": "line3"
    },
    "point1": {
        "color": "red",
        "marker": "o",
        "markersize": 6,
        "markeredgecolor": "black",
        "alpha": 1.0,
        "linestyle": "None",
        "label": "point1"
    },
    "point2": {
        "color": "orange",
        "marker": "o",
        "markersize": 4,
        "markeredgecolor": "None",
        "alpha": 0.5,
        "linestyle": "None",
        "label": "point2"
    },
    "point3": {
        "color": "None",
        "marker": "o",
        "markersize": 4,
        "markeredgecolor": "brown",
        "alpha": 1.0,
        "linestyle": "None",
        "label": "point3"
    }
}